/*****************************************************************************
 * audioscrobbler.c : audioscrobbler submission plugin
 *****************************************************************************
 * Copyright (C) 2006 Rafaël Carré
 * $Id: audioscrobbler.c 15846 2006-06-08 21:01:09Z Rafaël Carré $
 *
 * Authors: Rafaël Carré <rafael -dot- carre -at- wanadoo -dot- fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
 *****************************************************************************/

/*****************************************************************************
 * Preamble
 *****************************************************************************/
#include <stdlib.h>                                      /* malloc(), free() */
#include <errno.h>

#include <vlc/vlc.h>
#include <vlc/intf.h>
#include <vlc_meta.h>

/*****************************************************************************
 * Local prototypes
 *****************************************************************************/
static int  Open    ( vlc_object_t * );
static void Close   ( vlc_object_t * );
static void Run     ( intf_thread_t * );

static int ItemChange( vlc_object_t *, const char *,
                       vlc_value_t, vlc_value_t, void * );

static int AddToQueue ( intf_thread_t *p_this );
#define MAX_LENGTH 256

/*****************************************************************************
 * Module descriptor
 ****************************************************************************/


#define APPLICATION_NAME "VLC media player"

#define USERNAME_TEXT N_("Username")
#define USERNAME_LONGTEXT N_("Audioscrobbler username")

#define PASSWORD_TEXT N_("Password")
#define PASSWORD_LONGTEXT N_("Audioscrobbler password")

vlc_module_begin();
    set_category( CAT_INTERFACE );
    set_subcategory( SUBCAT_INTERFACE_CONTROL );
    set_shortname( N_( "Audioscrobbler" ) );
    set_description( _("Audioscrobbler submission Plugin") );

    add_string( "audioscrobbler-username", "test",NULL,
                USERNAME_TEXT, USERNAME_LONGTEXT, VLC_TRUE );

    add_string( "audioscrobbler-password", "pass",NULL,
                PASSWORD_TEXT, PASSWORD_LONGTEXT, VLC_TRUE );

    set_capability( "interface", 0 );
    set_callbacks( Open, Close );
vlc_module_end();

typedef struct audioscrobbler_song_t
{
    char        psz_a[MAX_LENGTH];  /* track artist     */
    char        psz_t[MAX_LENGTH];  /* track title      */
    char        psz_b[MAX_LENGTH];  /* track album      */
    int         i_l;                /* track length     */
/*  int         m;  */              /* musicbrainz id   */
    char        psz_i[20];          /* date             */
    vlc_bool_t  b_queued;           /* already in queue?*/
} audioscrobbler_song_t;

struct intf_sys_t
{
    audioscrobbler_song_t   **p_queue;
    int                     i_songs_nb;
    audioscrobbler_song_t   *p_current_song;
    char                    *psz_uptodate_url;
    char                    *psz_failed_reason;
    int                     i_interval;
    char                    *psz_submit_url;
    char                    *ps_md5_challenge;
    char                    *ps_md5_password;
};

#define CLIENT_NAME     tst /* change for vlc? */
#define CLIENT_VERSION  1.0

/*****************************************************************************
 * Open: initialize and create stuff
 *****************************************************************************/
 
static int Open( vlc_object_t *p_this )
{
    intf_thread_t *p_intf = (intf_thread_t *)p_this;

    playlist_t *p_playlist = (playlist_t *)vlc_object_find(
        p_intf, VLC_OBJECT_PLAYLIST, FIND_ANYWHERE );

    if( !p_playlist )
    {
        msg_Err( p_intf, "could not find playlist object" );
        return VLC_ENOOBJ;
    }

    p_intf->p_sys = malloc( sizeof( intf_sys_t ) );
    if( !p_intf->p_sys )
    {
        msg_Err( p_intf, "no memory" );
        return VLC_ENOMEM;
    }

    p_intf->p_sys->p_queue = calloc(10, sizeof( audioscrobbler_song_t ) );
    if( !p_intf->p_sys->p_queue )
    {
        msg_Err( p_intf, "no memory" );
        return VLC_ENOMEM;
    }

    p_intf->p_sys->p_current_song = malloc( sizeof( audioscrobbler_song_t ) );
    if( !p_intf->p_sys->p_current_song )
    {
        msg_Err( p_intf, "no memory" );
        return VLC_ENOMEM;
    }

    p_intf->p_sys->i_songs_nb = 0;
    p_intf->p_sys->p_current_song->b_queued = VLC_TRUE;

    var_AddCallback( p_playlist, "playlist-current", ItemChange, p_intf );

    vlc_object_release( p_playlist );

    p_intf->pf_run = Run;
    msg_Dbg( p_intf,"audioscrobbler plugin started");
    return VLC_SUCCESS;
}

/*****************************************************************************
 * Close: destroy interface stuff
 *****************************************************************************/
 /*
 virer la queue
 virer les vars
 free
 */
static void Close( vlc_object_t *p_this )
{
        intf_thread_t *p_intf = (intf_thread_t *)p_this;

        playlist_t *p_playlist = (playlist_t *)vlc_object_find(
        p_intf, VLC_OBJECT_PLAYLIST, FIND_ANYWHERE );

    if( p_playlist )
    {
        var_DelCallback( p_playlist, "playlist-current", ItemChange, p_intf );

        free( p_intf->p_sys->p_queue );
        
        free( p_intf->p_sys->p_current_song );

        free( p_intf->p_sys );
        
        vlc_object_release( p_playlist );
    }
}

/*****************************************************************************
 * Run
 *****************************************************************************/
static void Run( intf_thread_t *p_this )
{
    msleep( 100*INTF_IDLE_SLEEP );
}

/*****************************************************************************
 * ItemChange: Playlist item change callback
 *****************************************************************************/
static int ItemChange( vlc_object_t *p_this, const char *psz_var,
                       vlc_value_t oldval, vlc_value_t newval, void *p_data )
{
    playlist_t *p_playlist;
    char *psz_title = NULL;
    char *psz_artist = NULL;
    char *psz_album = NULL;
    int i_length = -1;
    time_t epoch;
    struct tm epoch_tm;
    char psz_date[20];

    intf_thread_t *p_intf = (intf_thread_t *)p_data;

    AddToQueue( p_data );

    input_thread_t *p_input=NULL;
    p_playlist = (playlist_t *)vlc_object_find( p_this, VLC_OBJECT_PLAYLIST,
                                                FIND_ANYWHERE );
    if( !p_playlist ) return VLC_EGENERIC;

    p_input = p_playlist->p_input;
    vlc_object_release( p_playlist );
    if( !p_input ) return VLC_SUCCESS;
    vlc_object_yield( p_input );

    if( p_input->b_dead || !p_input->input.p_item->psz_name )
    {
        /* Not playing anything ... */
        vlc_object_release( p_input );
        return VLC_SUCCESS;
    }

    /* Playing something ... */

    psz_artist = p_input->input.p_item->p_meta->psz_artist ?
                  strdup( p_input->input.p_item->p_meta->psz_artist ) :
                  strdup( _("no artist") );
    psz_album = p_input->input.p_item->p_meta->psz_album ?
                  strdup( p_input->input.p_item->p_meta->psz_album ) :
                  strdup( _("no album") );
    psz_title = strdup( p_input->input.p_item->psz_name );
    if( psz_title == NULL ) psz_title = strdup( N_("(no title)") );

    i_length = p_input->input.p_item->i_duration / 1000000;

    time( &epoch );
    gmtime_r( &epoch, &epoch_tm );
    snprintf( psz_date, 20, "%.4d-%.2d-%.2d %.2d:%.2d:%.2d",
    epoch_tm.tm_year+1900, epoch_tm.tm_mon+1, epoch_tm.tm_mday,
    epoch_tm.tm_hour+1, epoch_tm.tm_min+1, epoch_tm.tm_sec+1 );

    p_intf->p_sys->p_current_song->b_queued = VLC_FALSE;
    strncpy( p_intf->p_sys->p_current_song->psz_a, psz_artist, MAX_LENGTH );
    strncpy( p_intf->p_sys->p_current_song->psz_t, psz_title, MAX_LENGTH );
    strncpy( p_intf->p_sys->p_current_song->psz_b, psz_album, MAX_LENGTH );
    strncpy( p_intf->p_sys->p_current_song->psz_i, psz_date, 20 );
    p_intf->p_sys->p_current_song->i_l = i_length;

    free( psz_title );
    free( psz_artist );
    free( psz_album );

    vlc_object_release( p_input );
    return VLC_SUCCESS;
}
/*****************************************************************************
 * AddToQueue: Add the played song to the queue to be submitted
 *****************************************************************************/
static int AddToQueue ( intf_thread_t *p_this )
{
    intf_thread_t *p_intf = (intf_thread_t *)p_this;
    int i_songs_nb;
    
    if ( ( p_intf->p_sys->p_current_song->b_queued == VLC_TRUE ) ||
        ( p_intf->p_sys->p_current_song->i_l < 30 ) )
    {
        return VLC_SUCCESS;
    }
   
    i_songs_nb = p_intf->p_sys->i_songs_nb;

    if ( ( i_songs_nb < 0 ) || ( i_songs_nb >= 10 ) )
    {
        return VLC_EBADVAR;
    }

    memcpy( &p_intf->p_sys->p_queue[i_songs_nb], p_intf->p_sys->p_current_song,
        sizeof( audioscrobbler_song_t ) );
    p_intf->p_sys->i_songs_nb++;
    p_intf->p_sys->p_current_song->b_queued = VLC_TRUE;
    return VLC_SUCCESS;
}
